# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: message.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import schema_pb2 as schema__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\rmessage.proto\x12\x15relationalai.protocol\x1a\x0cschema.proto\"J\n\x0cMetadataInfo\x12:\n\trelations\x18\x01 \x03(\x0b\x32\'.relationalai.protocol.RelationMetadata\"]\n\x10RelationMetadata\x12\x36\n\x0brelation_id\x18\x01 \x01(\x0b\x32!.relationalai.protocol.RelationId\x12\x11\n\tfile_name\x18\x02 \x01(\tb\x06proto3')



_METADATAINFO = DESCRIPTOR.message_types_by_name['MetadataInfo']
_RELATIONMETADATA = DESCRIPTOR.message_types_by_name['RelationMetadata']
MetadataInfo = _reflection.GeneratedProtocolMessageType('MetadataInfo', (_message.Message,), {
  'DESCRIPTOR' : _METADATAINFO,
  '__module__' : 'message_pb2'
  # @@protoc_insertion_point(class_scope:relationalai.protocol.MetadataInfo)
  })
_sym_db.RegisterMessage(MetadataInfo)

RelationMetadata = _reflection.GeneratedProtocolMessageType('RelationMetadata', (_message.Message,), {
  'DESCRIPTOR' : _RELATIONMETADATA,
  '__module__' : 'message_pb2'
  # @@protoc_insertion_point(class_scope:relationalai.protocol.RelationMetadata)
  })
_sym_db.RegisterMessage(RelationMetadata)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _METADATAINFO._serialized_start=54
  _METADATAINFO._serialized_end=128
  _RELATIONMETADATA._serialized_start=130
  _RELATIONMETADATA._serialized_end=223
# @@protoc_insertion_point(module_scope)
