# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: schema.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0cschema.proto\x12\x15relationalai.protocol\"?\n\nRelationId\x12\x31\n\targuments\x18\x01 \x03(\x0b\x32\x1e.relationalai.protocol.RelType\"D\n\x08RelTuple\x12\x38\n\targuments\x18\x01 \x03(\x0b\x32%.relationalai.protocol.PrimitiveValue\".\n\tRelInt128\x12\x10\n\x08highbits\x18\x01 \x01(\x04\x12\x0f\n\x07lowbits\x18\x02 \x01(\x04\"/\n\nRelUInt128\x12\x10\n\x08highbits\x18\x01 \x01(\x04\x12\x0f\n\x07lowbits\x18\x02 \x01(\x04\"\xeb\x03\n\x0ePrimitiveValue\x12\x31\n\x03tag\x18\x01 \x01(\x0e\x32$.relationalai.protocol.PrimitiveType\x12\x36\n\nint128_val\x18\x02 \x01(\x0b\x32 .relationalai.protocol.RelInt128H\x00\x12\x13\n\tint64_val\x18\x03 \x01(\x03H\x00\x12\x13\n\tint32_val\x18\x04 \x01(\x05H\x00\x12\x13\n\tint16_val\x18\x05 \x01(\x05H\x00\x12\x12\n\x08int8_val\x18\x06 \x01(\x05H\x00\x12\x38\n\x0buint128_val\x18\x07 \x01(\x0b\x32!.relationalai.protocol.RelUInt128H\x00\x12\x14\n\nuint64_val\x18\x08 \x01(\x04H\x00\x12\x14\n\nuint32_val\x18\t \x01(\rH\x00\x12\x14\n\nuint16_val\x18\n \x01(\rH\x00\x12\x13\n\tuint8_val\x18\x0b \x01(\rH\x00\x12\x15\n\x0b\x66loat64_val\x18\x0c \x01(\x01H\x00\x12\x15\n\x0b\x66loat32_val\x18\r \x01(\x02H\x00\x12\x15\n\x0b\x66loat16_val\x18\x0e \x01(\x02H\x00\x12\x12\n\x08\x63har_val\x18\x0f \x01(\rH\x00\x12\x12\n\x08\x62ool_val\x18\x10 \x01(\x08H\x00\x12\x14\n\nstring_val\x18\x11 \x01(\x0cH\x00\x42\x07\n\x05value\"C\n\tValueType\x12\x36\n\x0e\x61rgument_types\x18\x01 \x03(\x0b\x32\x1e.relationalai.protocol.RelType\"p\n\x0c\x43onstantType\x12\x30\n\x08rel_type\x18\x01 \x01(\x0b\x32\x1e.relationalai.protocol.RelType\x12.\n\x05value\x18\x02 \x01(\x0b\x32\x1f.relationalai.protocol.RelTuple\"\xe3\x01\n\x07RelType\x12(\n\x03tag\x18\x01 \x01(\x0e\x32\x1b.relationalai.protocol.Kind\x12<\n\x0eprimitive_type\x18\x02 \x01(\x0e\x32$.relationalai.protocol.PrimitiveType\x12\x34\n\nvalue_type\x18\x03 \x01(\x0b\x32 .relationalai.protocol.ValueType\x12:\n\rconstant_type\x18\x04 \x01(\x0b\x32#.relationalai.protocol.ConstantType*\x92\x02\n\rPrimitiveType\x12\x14\n\x10UNSPECIFIED_TYPE\x10\x00\x12\x0b\n\x07INT_128\x10\x01\x12\n\n\x06INT_64\x10\x02\x12\n\n\x06INT_32\x10\x03\x12\n\n\x06INT_16\x10\x04\x12\t\n\x05INT_8\x10\x05\x12\x0c\n\x08UINT_128\x10\x06\x12\x0b\n\x07UINT_64\x10\x07\x12\x0b\n\x07UINT_32\x10\x08\x12\x0b\n\x07UINT_16\x10\t\x12\n\n\x06UINT_8\x10\n\x12\x0c\n\x08\x46LOAT_64\x10\x0b\x12\x0c\n\x08\x46LOAT_32\x10\x0c\x12\x0c\n\x08\x46LOAT_16\x10\r\x12\x08\n\x04\x43HAR\x10\x0e\x12\x08\n\x04\x42OOL\x10\x0f\x12\n\n\x06STRING\x10\x10\x12\n\n\x06SYMBOL\x10\x11\x12\x18\n\x14VARIABLE_SIZE_STRING\x10\x12*S\n\x04Kind\x12\x14\n\x10UNSPECIFIED_KIND\x10\x00\x12\x12\n\x0ePRIMITIVE_TYPE\x10\x01\x12\x0e\n\nVALUE_TYPE\x10\x02\x12\x11\n\rCONSTANT_TYPE\x10\x03\x62\x06proto3')

_PRIMITIVETYPE = DESCRIPTOR.enum_types_by_name['PrimitiveType']
PrimitiveType = enum_type_wrapper.EnumTypeWrapper(_PRIMITIVETYPE)
_KIND = DESCRIPTOR.enum_types_by_name['Kind']
Kind = enum_type_wrapper.EnumTypeWrapper(_KIND)
UNSPECIFIED_TYPE = 0
INT_128 = 1
INT_64 = 2
INT_32 = 3
INT_16 = 4
INT_8 = 5
UINT_128 = 6
UINT_64 = 7
UINT_32 = 8
UINT_16 = 9
UINT_8 = 10
FLOAT_64 = 11
FLOAT_32 = 12
FLOAT_16 = 13
CHAR = 14
BOOL = 15
STRING = 16
SYMBOL = 17
VARIABLE_SIZE_STRING = 18
UNSPECIFIED_KIND = 0
PRIMITIVE_TYPE = 1
VALUE_TYPE = 2
CONSTANT_TYPE = 3


_RELATIONID = DESCRIPTOR.message_types_by_name['RelationId']
_RELTUPLE = DESCRIPTOR.message_types_by_name['RelTuple']
_RELINT128 = DESCRIPTOR.message_types_by_name['RelInt128']
_RELUINT128 = DESCRIPTOR.message_types_by_name['RelUInt128']
_PRIMITIVEVALUE = DESCRIPTOR.message_types_by_name['PrimitiveValue']
_VALUETYPE = DESCRIPTOR.message_types_by_name['ValueType']
_CONSTANTTYPE = DESCRIPTOR.message_types_by_name['ConstantType']
_RELTYPE = DESCRIPTOR.message_types_by_name['RelType']
RelationId = _reflection.GeneratedProtocolMessageType('RelationId', (_message.Message,), {
  'DESCRIPTOR' : _RELATIONID,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:relationalai.protocol.RelationId)
  })
_sym_db.RegisterMessage(RelationId)

RelTuple = _reflection.GeneratedProtocolMessageType('RelTuple', (_message.Message,), {
  'DESCRIPTOR' : _RELTUPLE,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:relationalai.protocol.RelTuple)
  })
_sym_db.RegisterMessage(RelTuple)

RelInt128 = _reflection.GeneratedProtocolMessageType('RelInt128', (_message.Message,), {
  'DESCRIPTOR' : _RELINT128,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:relationalai.protocol.RelInt128)
  })
_sym_db.RegisterMessage(RelInt128)

RelUInt128 = _reflection.GeneratedProtocolMessageType('RelUInt128', (_message.Message,), {
  'DESCRIPTOR' : _RELUINT128,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:relationalai.protocol.RelUInt128)
  })
_sym_db.RegisterMessage(RelUInt128)

PrimitiveValue = _reflection.GeneratedProtocolMessageType('PrimitiveValue', (_message.Message,), {
  'DESCRIPTOR' : _PRIMITIVEVALUE,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:relationalai.protocol.PrimitiveValue)
  })
_sym_db.RegisterMessage(PrimitiveValue)

ValueType = _reflection.GeneratedProtocolMessageType('ValueType', (_message.Message,), {
  'DESCRIPTOR' : _VALUETYPE,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:relationalai.protocol.ValueType)
  })
_sym_db.RegisterMessage(ValueType)

ConstantType = _reflection.GeneratedProtocolMessageType('ConstantType', (_message.Message,), {
  'DESCRIPTOR' : _CONSTANTTYPE,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:relationalai.protocol.ConstantType)
  })
_sym_db.RegisterMessage(ConstantType)

RelType = _reflection.GeneratedProtocolMessageType('RelType', (_message.Message,), {
  'DESCRIPTOR' : _RELTYPE,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:relationalai.protocol.RelType)
  })
_sym_db.RegisterMessage(RelType)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _PRIMITIVETYPE._serialized_start=1179
  _PRIMITIVETYPE._serialized_end=1453
  _KIND._serialized_start=1455
  _KIND._serialized_end=1538
  _RELATIONID._serialized_start=39
  _RELATIONID._serialized_end=102
  _RELTUPLE._serialized_start=104
  _RELTUPLE._serialized_end=172
  _RELINT128._serialized_start=174
  _RELINT128._serialized_end=220
  _RELUINT128._serialized_start=222
  _RELUINT128._serialized_end=269
  _PRIMITIVEVALUE._serialized_start=272
  _PRIMITIVEVALUE._serialized_end=763
  _VALUETYPE._serialized_start=765
  _VALUETYPE._serialized_end=832
  _CONSTANTTYPE._serialized_start=834
  _CONSTANTTYPE._serialized_end=946
  _RELTYPE._serialized_start=949
  _RELTYPE._serialized_end=1176
# @@protoc_insertion_point(module_scope)
